from typing import List

location_history_files_editable_columns: List[str] = [
    "Ignore",
    "Location history provider",
    "Content type",
]

location_history_by_date_editable_columns: List[str] = ["Trip"]

"""
travel_reports_editable_columns = [
    "Include in expense report",
    "Expense report receiver",
    "Expense report",
    "Expense report accounting period",
    "Destination",
    "Travel purpose",
    "Left date & time",
    "Returned date & time",
    "Other comment",
    "Days Of Personal Vacation",
    "Private Car Kilometers",
    "Days With Paid Breakfast",
    "Days With Paid Lunch",
    "Days With Paid Dinner",
]
"""
