from typing import List

default_transaction_files_editable_columns: List[str] = [
    "Ignore",
    "Account provider",
    "Account",
    "Content type",
    "Account currency",
]

default_transactions_editable_columns: List[str] = [
    "Clarification",
    "Type",
    "Income/Expense Category",
    "Currency",
]
